@@ -1,4 +1,5 @@
 /*
+ * Copyright(c) 2022 ZettaScale Technology and others
  * Copyright(c) 2006 to 2018 ADLINK Technology Limited and others
  *
  * This program and the accompanying materials are made available under the
@@ -24,6 +25,7 @@
 #include <stdint.h>
 #include <stdbool.h>
 #include "dds/export.h"
+#include "dds/features.h"
 #include "dds/ddsc/dds_public_alloc.h"
 #include "dds/ddsc/dds_opcodes.h"
 
@@ -43,7 +45,8 @@ dds_sequence_t;
 typedef struct dds_key_descriptor
 {
   const char * m_name;
-  uint32_t m_index;
+  uint32_t m_offset;
+  uint32_t m_idx;
 }
 dds_key_descriptor_t;
 
@@ -53,6 +56,21 @@ dds_key_descriptor_t;
   API is a pointer to the "topic_descriptor_t" struct type.
 */
 
+struct dds_type_meta_ser
+{
+  unsigned char * data;
+  uint32_t sz;
+};
+
+#define DDS_DATA_REPRESENTATION_XCDR1    0
+#define DDS_DATA_REPRESENTATION_XML      1
+#define DDS_DATA_REPRESENTATION_XCDR2    2
+
+#define DDS_DATA_REPRESENTATION_FLAG_XCDR1  (1u << DDS_DATA_REPRESENTATION_XCDR1)
+#define DDS_DATA_REPRESENTATION_FLAG_XML    (1u << DDS_DATA_REPRESENTATION_XML)
+#define DDS_DATA_REPRESENTATION_FLAG_XCDR2  (1u << DDS_DATA_REPRESENTATION_XCDR2)
+#define DDS_DATA_REPRESENTATION_RESTRICT_DEFAULT  (DDS_DATA_REPRESENTATION_FLAG_XCDR1 | DDS_DATA_REPRESENTATION_FLAG_XCDR2)
+
 typedef struct dds_topic_descriptor
 {
   const uint32_t m_size;               /* Size of topic type */
@@ -64,6 +82,11 @@ typedef struct dds_topic_descriptor
   const uint32_t m_nops;               /* Number of ops in m_ops */
   const uint32_t * m_ops;              /* Marshalling meta data */
   const char * m_meta;                 /* XML topic description meta data */
+  struct dds_type_meta_ser type_information;  /* XCDR2 serialized TypeInformation, only present if flag DDS_TOPIC_XTYPES_METADATA is set */
+  struct dds_type_meta_ser type_mapping;      /* XCDR2 serialized TypeMapping: maps type-id to type object and minimal to complete type id,
+                                                   only present if flag DDS_TOPIC_XTYPES_METADATA is set */
+  const uint32_t restrict_data_representation; /* restrictions on the data representations allowed for the top-level type for this topic,
+                                           only present if flag DDS_TOPIC_RESTRICT_DATA_REPRESENTATION */
 }
 dds_topic_descriptor_t;
 
@@ -122,6 +145,14 @@ typedef enum dds_find_scope
 }
 dds_find_scope_t;
 
+/* Type identifier kind for getting endpoint type identifier */
+typedef enum dds_typeid_kind
+{
+  DDS_TYPEID_MINIMAL,
+  DDS_TYPEID_COMPLETE
+}
+dds_typeid_kind_t;
+
 /**
  * Description : Enable or disable write batching. Overrides default configuration
  * setting for write batching (Internal/WriteBatch).
