@@ -28,6 +28,7 @@
 #include "dds/ddsi/ddsi_xqos.h"
 #include "dds/ddsi/q_transmit.h"
 #include "dds/ddsi/q_bswap.h"
+#include "dds/ddsi/q_entity.h"
 
 DDS_EXPORT extern inline dds_entity *dds_entity_from_handle_link (struct dds_handle_link *hdllink);
 DDS_EXPORT extern inline bool dds_entity_is_enabled (const dds_entity *e);
@@ -259,6 +260,11 @@ dds_entity_t dds_entity_init (dds_entity *e, dds_entity *parent, dds_entity_kind
   dds_reset_listener (&e->m_listener);
   if (listener)
     dds_merge_listener (&e->m_listener, listener);
+
+  /* Special case: the on_data_on_readers event doesn't exist on DataReaders. */
+  if (kind == DDS_KIND_READER)
+    e->m_listener.on_data_on_readers = 0;
+
   if (parent)
   {
     ddsrt_mutex_lock (&parent->m_observers_lock);
@@ -720,7 +726,7 @@ dds_return_t dds_get_qos (dds_entity_t entity, dds_qos_t *qos)
     }
 
     dds_reset_qos (qos);
-    ddsi_xqos_mergein_missing (qos, entity_qos, ~(QP_TOPIC_NAME | QP_TYPE_NAME | QP_CYCLONE_TYPE_INFORMATION));
+    ddsi_xqos_mergein_missing (qos, entity_qos, ~(QP_TOPIC_NAME | QP_TYPE_NAME | QP_TYPE_INFORMATION));
     ret = DDS_RETCODE_OK;
   }
   dds_entity_unlock(e);
@@ -1032,6 +1038,11 @@ dds_return_t dds_set_listener (dds_entity_t entity, const dds_listener_t *listen
   dds_reset_listener (&e->m_listener);
   if (listener)
     dds_merge_listener (&e->m_listener, listener);
+
+  /* Special case: the on_data_on_readers event doesn't exist on DataReaders. */
+  if (dds_entity_kind (e) == DDS_KIND_READER)
+    e->m_listener.on_data_on_readers = 0;
+
   x = e;
   while (dds_entity_kind (x) != DDS_KIND_CYCLONEDDS)
   {
@@ -1040,6 +1051,7 @@ dds_return_t dds_set_listener (dds_entity_t entity, const dds_listener_t *listen
     dds_inherit_listener (&e->m_listener, &x->m_listener);
     ddsrt_mutex_unlock (&x->m_observers_lock);
   }
+
   ddsrt_mutex_unlock (&e->m_observers_lock);
   pushdown_listener (e);
   dds_entity_unpin (e);
@@ -1462,7 +1474,7 @@ dds_entity_t dds_get_topic (dds_entity_t entity)
     }
     case DDS_KIND_WRITER: {
       dds_writer *wr = (dds_writer *) e;
-      assert (dds__get_builtin_topic_pseudo_handle_from_typename (wr->m_topic->m_stype->type_name) < 0);
+      assert (dds__get_builtin_topic_pseudo_handle_from_typename (wr->m_wr->type->type_name) < 0);
       hdl = wr->m_topic->m_entity.m_hdllink.hdl;
       break;
     }
