@@ -16,7 +16,7 @@
 #include "dds/ddsrt/heap.h"
 #include "dds/ddsi/ddsi_serdata.h"
 #include "dds/ddsi/q_unused.h"
-#include "dds/ddsi/q_config.h"
+#include "dds/ddsi/ddsi_config_impl.h"
 #include "dds/ddsi/ddsi_entity_index.h"
 #include "dds/ddsi/q_entity.h"
 #include "dds/ddsi/ddsi_domaingv.h"
@@ -227,8 +227,8 @@ static bool bwhc_sample_iter_borrow_next (struct whc_sample_iter *opaque_it, str
 static void bwhc_get_state (const struct whc *whc, struct whc_state *st)
 {
   (void)whc;
-  st->max_seq = -1;
-  st->min_seq = -1;
+  st->max_seq = 0;
+  st->min_seq = 0;
   st->unacked_bytes = 0;
 }
 
