@@ -158,12 +158,9 @@ static void test_pmd_count(dds_liveliness_kind_t kind, uint32_t ldur, double mul
   dds_attach_t triggered;
   uint32_t status;
   char name[100];
-  dds_time_t t;
 
-  t = dds_time();
-  printf("%d.%06d running test: kind %s, lease duration %"PRIu32", delay %d, %s reader\n",
-         (int32_t)(t / DDS_NSECS_IN_SEC), (int32_t)(t % DDS_NSECS_IN_SEC) / 1000,
-         kind == 0 ? "A" : "MP", ldur, (int32_t)(mult * ldur), remote_reader ? "remote" : "local");
+  tprintf("running test: kind %s, lease duration %"PRIu32", delay %d, %s reader\n",
+          kind == 0 ? "A" : "MP", ldur, (int32_t)(mult * ldur), remote_reader ? "remote" : "local");
 
   /* wait for initial PMD to be sent by the participant */
   while (get_pmd_seqno(g_pub_participant) < 1)
@@ -201,10 +198,7 @@ static void test_pmd_count(dds_liveliness_kind_t kind, uint32_t ldur, double mul
   dds_sleepfor(DDS_MSECS((dds_duration_t)(mult * ldur)));
   end_seqno = get_pmd_seqno(g_pub_participant);
 
-  t = dds_time();
-  printf("%d.%06d PMD sequence no: start %" PRId64 " -> end %" PRId64 "\n",
-         (int32_t)(t / DDS_NSECS_IN_SEC), (int32_t)(t % DDS_NSECS_IN_SEC) / 1000,
-         start_seqno, end_seqno);
+  tprintf("PMD sequence no: start %" PRIu64 " -> end %" PRIu64 "\n", start_seqno, end_seqno);
 
   /* End-start should be mult - 1 under ideal circumstances, but consider the test successful
            when at least 50% of the expected PMD's was sent. This checks that the frequency for sending
@@ -258,9 +252,8 @@ static void test_expire_liveliness_kinds(uint32_t ldur, double mult, uint32_t wr
   do
   {
     tstart = dds_time();
-    printf("%d.%06d running test: lease duration %"PRIu32", delay %f, auto/man-by-part/man-by-topic %"PRIu32"/%"PRIu32"/%"PRIu32", %s reader\n",
-           (int32_t)(tstart / DDS_NSECS_IN_SEC), (int32_t)(tstart % DDS_NSECS_IN_SEC) / 1000,
-           ldur, mult, wr_cnt_auto, wr_cnt_man_pp, wr_cnt_man_tp, remote_reader ? "remote" : "local");
+    tprintf("running test: lease duration %"PRIu32", delay %f, auto/man-by-part/man-by-topic %"PRIu32"/%"PRIu32"/%"PRIu32", %s reader\n",
+            ldur, mult, wr_cnt_auto, wr_cnt_man_pp, wr_cnt_man_tp, remote_reader ? "remote" : "local");
 
     /* topics */
     create_unique_topic_name("ddsc_liveliness_expire_kinds", name, sizeof name);
@@ -303,14 +296,13 @@ static void test_expire_liveliness_kinds(uint32_t ldur, double mult, uint32_t wr
     if (t - tstart > DDS_MSECS(ldur) / 2)
     {
       ldur *= 10 / (run + 1);
-      printf("%d.%06d failed to create writers in time\n",
-             (int32_t)(t / DDS_NSECS_IN_SEC), (int32_t)(t % DDS_NSECS_IN_SEC) / 1000);
+      tprintf("failed to create writers in time\n");
     }
     else
     {
       /* check alive count before proxy writers are expired */
       dds_get_liveliness_changed_status(reader, &lstatus);
-      printf("%d.%06d writers alive: %"PRIu32"\n", (int32_t)(t / DDS_NSECS_IN_SEC), (int32_t)(t % DDS_NSECS_IN_SEC) / 1000, lstatus.alive_count);
+      tprintf("writers alive: %"PRIu32"\n", lstatus.alive_count);
       CU_ASSERT_EQUAL_FATAL(lstatus.alive_count, wr_cnt);
 
       dds_time_t tstop = tstart + DDS_MSECS((dds_duration_t)(mult * ldur));
@@ -322,24 +314,20 @@ static void test_expire_liveliness_kinds(uint32_t ldur, double mult, uint32_t wr
         CU_ASSERT_EQUAL_FATAL(dds_get_liveliness_changed_status(reader, &lstatus), DDS_RETCODE_OK);
         stopped += (uint32_t)lstatus.not_alive_count_change;
       } while (dds_time() < tstop);
-      t = dds_time();
-      printf("%d.%06d writers stopped: %u\n",
-             (int32_t)(t / DDS_NSECS_IN_SEC), (int32_t)(t % DDS_NSECS_IN_SEC) / 1000, stopped);
+      tprintf("writers stopped: %u\n", stopped);
 
       size_t exp_stopped = mult < 1 ? 0 : (wr_cnt_man_pp + wr_cnt_man_tp);
       size_t exp_alive = mult < 1 ? wr_cnt : wr_cnt_auto;
       CU_ASSERT_EQUAL_FATAL(dds_get_liveliness_changed_status(reader, &lstatus), DDS_RETCODE_OK);
-      printf("%d.%06d writers alive: %u (exp: %u) not-alive: %u (exp %u)\n",
-             (int32_t)(t / DDS_NSECS_IN_SEC), (int32_t)(t % DDS_NSECS_IN_SEC) / 1000,
-             lstatus.alive_count, (unsigned) exp_alive,
-             lstatus.not_alive_count, (unsigned) exp_stopped);
+      tprintf("writers alive: %u (exp: %u) not-alive: %u (exp %u)\n",
+              lstatus.alive_count, (unsigned) exp_alive,
+              lstatus.not_alive_count, (unsigned) exp_stopped);
       if (stopped == exp_stopped && lstatus.alive_count == exp_alive)
         test_finished = true;
       else
       {
         ldur *= 10 / (run + 1);
-        printf("%d.%06d incorrect number of stopped/alive writers\n",
-               (int32_t)(t / DDS_NSECS_IN_SEC), (int32_t)(t % DDS_NSECS_IN_SEC) / 1000);
+        tprintf("incorrect number of stopped/alive writers\n");
       }
     }
 
@@ -359,15 +347,14 @@ static void test_expire_liveliness_kinds(uint32_t ldur, double mult, uint32_t wr
     {
       if (++run > 3)
       {
-        printf("%d.%06d run limit reached, test failed\n", (int32_t)(tstart / DDS_NSECS_IN_SEC), (int32_t)(tstart % DDS_NSECS_IN_SEC) / 1000);
+        tprintf("run limit reached, test failed\n");
         CU_FAIL_FATAL("Run limit reached");
         test_finished = true;
         continue;
       }
       else
       {
-        printf("%d.%06d restarting test with ldur %"PRIu32"\n",
-               (int32_t)(t / DDS_NSECS_IN_SEC), (int32_t)(t % DDS_NSECS_IN_SEC) / 1000, ldur);
+        tprintf("restarting test with ldur %"PRIu32"\n", ldur);
       }
     }
   } while (!test_finished);
@@ -478,7 +465,7 @@ static void test_lease_duration_pwr(bool remote_reader)
   uint32_t status;
   dds_duration_t ldur;
 
-  printf("running test lease_duration_pwr: %s reader\n", remote_reader ? "remote" : "local");
+  tprintf("running test lease_duration_pwr: %s reader\n", remote_reader ? "remote" : "local");
 
   /* topics */
   create_unique_topic_name("ddsc_liveliness_ldurpwr", name, sizeof name);
@@ -557,7 +544,7 @@ static void test_create_delete_writer_stress(bool remote_reader)
   Space_Type1 sample = {0, 0, 0};
   int64_t ldur = 1000;
 
-  printf("running test create_delete_writer_stress: %s reader\n", remote_reader ? "remote" : "local");
+  tprintf("running test create_delete_writer_stress: %s reader\n", remote_reader ? "remote" : "local");
 
   /* topics */
   create_unique_topic_name("ddsc_liveliness_wr_stress", name, sizeof name);
@@ -595,21 +582,21 @@ static void test_create_delete_writer_stress(bool remote_reader)
       alive_writers_man++;
   }
   dds_delete_qos(wqos);
-  printf("%"PRId64" alive_writers_auto: %"PRIu32", alive_writers_man: %"PRIu32"\n", dds_time(), alive_writers_auto, alive_writers_man);
+  tprintf("%"PRId64" alive_writers_auto: %"PRIu32", alive_writers_man: %"PRIu32"\n", dds_time(), alive_writers_auto, alive_writers_man);
 
   /* wait for auto liveliness writers to become alive and manual-by-pp writers to become not-alive */
   do
   {
     CU_ASSERT_EQUAL_FATAL(dds_get_liveliness_changed_status(reader, &lstatus), DDS_RETCODE_OK);
-    printf("%"PRId64" alive: %"PRIu32", not-alive: %"PRIu32"\n", dds_time(), lstatus.alive_count, lstatus.not_alive_count);
+    tprintf("%"PRId64" alive: %"PRIu32", not-alive: %"PRIu32"\n", dds_time(), lstatus.alive_count, lstatus.not_alive_count);
     dds_sleepfor(DDS_MSECS(50));
   } while (lstatus.alive_count != alive_writers_auto || lstatus.not_alive_count != alive_writers_man);
 
   /* check that counts are stable after a delay */
-  printf("%"PRId64" wait for half ldur (%"PRId64"ms)\n", dds_time(), ldur);
+  tprintf("%"PRId64" wait for half ldur (%"PRId64"ms)\n", dds_time(), ldur);
   dds_sleepfor(DDS_MSECS(ldur / 2));
   CU_ASSERT_EQUAL_FATAL(dds_get_liveliness_changed_status(reader, &lstatus), DDS_RETCODE_OK);
-  printf("%"PRId64" alive: %"PRIu32", not-alive: %"PRIu32"\n", dds_time(), lstatus.alive_count, lstatus.not_alive_count);
+  tprintf("%"PRId64" alive: %"PRIu32", not-alive: %"PRIu32"\n", dds_time(), lstatus.alive_count, lstatus.not_alive_count);
   CU_ASSERT_FATAL(lstatus.alive_count == alive_writers_auto && lstatus.not_alive_count == alive_writers_man);
 
   /* cleanup remaining writers */
@@ -622,7 +609,7 @@ static void test_create_delete_writer_stress(bool remote_reader)
   do
   {
     CU_ASSERT_EQUAL_FATAL(dds_get_liveliness_changed_status(reader, &lstatus), DDS_RETCODE_OK);
-    printf("%"PRId64" alive: %"PRIu32", not: %"PRIu32"\n", dds_time(), lstatus.alive_count, lstatus.not_alive_count);
+    tprintf("%"PRId64" alive: %"PRIu32", not: %"PRIu32"\n", dds_time(), lstatus.alive_count, lstatus.not_alive_count);
     dds_sleepfor(DDS_MSECS(ldur / 10));
   } while (lstatus.alive_count > 0 || lstatus.not_alive_count > 0);
   CU_ASSERT_EQUAL_FATAL(dds_waitset_detach(waitset, reader), DDS_RETCODE_OK);
@@ -660,7 +647,7 @@ static void test_status_counts(bool remote_reader)
   dds_duration_t ldur = DDS_MSECS(500);
   Space_Type1 sample = {1, 0, 0};
 
-  printf("running test status_counts: %s reader\n", remote_reader ? "remote" : "local");
+  tprintf("running test status_counts: %s reader\n", remote_reader ? "remote" : "local");
 
   /* topics */
   create_unique_topic_name("ddsc_liveliness_status_counts", name, sizeof name);
@@ -759,8 +746,8 @@ static void test_assert_liveliness(uint32_t wr_cnt_auto, uint32_t wr_cnt_man_pp,
   {
     wr_cnt = 0;
     assert(wr_cnt_auto + wr_cnt_man_pp + wr_cnt_man_tp < MAX_WRITERS);
-    printf("running test assert_liveliness: auto/man-by-part/man-by-topic %"PRIu32"/%"PRIu32"/%"PRIu32" with ldur %"PRIu32", %s reader\n",
-           wr_cnt_auto, wr_cnt_man_pp, wr_cnt_man_tp, ldur, remote_reader ? "remote" : "local");
+    tprintf("running test assert_liveliness: auto/man-by-part/man-by-topic %"PRIu32"/%"PRIu32"/%"PRIu32" with ldur %"PRIu32", %s reader\n",
+            wr_cnt_auto, wr_cnt_man_pp, wr_cnt_man_tp, ldur, remote_reader ? "remote" : "local");
 
     /* topics */
     create_unique_topic_name("ddsc_liveliness_assert", name, sizeof name);
@@ -787,8 +774,7 @@ static void test_assert_liveliness(uint32_t wr_cnt_auto, uint32_t wr_cnt_man_pp,
     if (t - tstart > DDS_MSECS(ldur) / 2)
     {
       ldur *= 10 / (run + 1);
-      printf("%d.%06d failed to create writers with non-automatic liveliness kind in time\n",
-             (int32_t)(t / DDS_NSECS_IN_SEC), (int32_t)(t % DDS_NSECS_IN_SEC) / 1000);
+      tprintf("failed to create writers with non-automatic liveliness kind in time\n");
     }
     else
     {
@@ -809,11 +795,11 @@ static void test_assert_liveliness(uint32_t wr_cnt_auto, uint32_t wr_cnt_man_pp,
         dds_sleepfor(DDS_MSECS(50));
       } while (dds_time() < tstop);
       CU_ASSERT_EQUAL_FATAL(dds_get_liveliness_changed_status(reader, &lstatus), DDS_RETCODE_OK);
-      printf("writers alive with dds_assert_liveliness on all writers: %"PRIu32", writers stopped: %"PRIu32"\n", lstatus.alive_count, stopped);
+      tprintf("writers alive with dds_assert_liveliness on all writers: %"PRIu32", writers stopped: %"PRIu32"\n", lstatus.alive_count, stopped);
       if (lstatus.alive_count != wr_cnt_auto + wr_cnt_man_pp + wr_cnt_man_tp || stopped != 0)
       {
         ldur *= 10 / (run + 1);
-        printf("incorrect number of writers alive or stopped writers\n");
+        tprintf("incorrect number of writers alive or stopped writers\n");
       }
       else
       {
@@ -830,11 +816,11 @@ static void test_assert_liveliness(uint32_t wr_cnt_auto, uint32_t wr_cnt_man_pp,
           dds_sleepfor(DDS_MSECS(50));
         } while (dds_time() < tstop);
         dds_get_liveliness_changed_status(reader, &lstatus);
-        printf("writers alive with dds_assert_liveliness on participant: %"PRIu32", writers stopped: %"PRIu32"\n", lstatus.alive_count, stopped);
+        tprintf("writers alive with dds_assert_liveliness on participant: %"PRIu32", writers stopped: %"PRIu32"\n", lstatus.alive_count, stopped);
         if (lstatus.alive_count != wr_cnt_auto + wr_cnt_man_pp || stopped != wr_cnt_man_tp)
         {
           ldur *= 10 / (run + 1);
-          printf("incorrect number of writers alive or stopped writers\n");
+          tprintf("incorrect number of writers alive or stopped writers\n");
         }
         else
         {
@@ -861,7 +847,7 @@ static void test_assert_liveliness(uint32_t wr_cnt_auto, uint32_t wr_cnt_man_pp,
       }
       else
       {
-        printf("restarting test with ldur %"PRIu32"\n", ldur);
+        tprintf("restarting test with ldur %"PRIu32"\n", ldur);
       }
     }
   } while (!test_finished);
@@ -894,12 +880,12 @@ static void liveliness_changed_listener (dds_entity_t rd, const dds_liveliness_c
   {
     if (st->w0_handle == 0)
     {
-      printf ("liveliness_changed_listener: w0 = %"PRIx64"\n", status.last_publication_handle);
+      tprintf ("liveliness_changed_listener: w0 = %"PRIx64"\n", status.last_publication_handle);
       st->w0_handle = status.last_publication_handle;
     }
     else
     {
-      printf ("liveliness_changed_listener: too many writer handles\n");
+      tprintf ("liveliness_changed_listener: too many writer handles\n");
       st->weirdness = true;
     }
   }
@@ -915,12 +901,12 @@ static void liveliness_changed_listener (dds_entity_t rd, const dds_liveliness_c
           st->w0_alive++;
         else
         {
-          printf ("liveliness_changed_listener: alive_count_change = %d: unrecognized writer\n", status.alive_count_change);
+          tprintf ("liveliness_changed_listener: alive_count_change = %d: unrecognized writer\n", status.alive_count_change);
           st->weirdness = true;
         }
         break;
       default:
-        printf ("liveliness_changed_listener: alive_count_change = %d\n", status.alive_count_change);
+        tprintf ("liveliness_changed_listener: alive_count_change = %d\n", status.alive_count_change);
         st->weirdness = true;
     }
 
@@ -933,18 +919,18 @@ static void liveliness_changed_listener (dds_entity_t rd, const dds_liveliness_c
           st->w0_not_alive++;
         else
         {
-          printf ("liveliness_changed_listener: not_alive_count_change = %d: unrecognized writer\n", status.not_alive_count_change);
+          tprintf ("liveliness_changed_listener: not_alive_count_change = %d: unrecognized writer\n", status.not_alive_count_change);
           st->weirdness = true;
         }
         break;
       default:
-        printf ("liveliness_changed_listener: not_alive_count_change = %d\n", status.not_alive_count_change);
+        tprintf ("liveliness_changed_listener: not_alive_count_change = %d\n", status.not_alive_count_change);
         st->weirdness = true;
     }
   }
   else
   {
-    printf ("liveliness_changed_listener: alive_count_change = 0 && not_alive_count_change = 0\n");
+    tprintf ("liveliness_changed_listener: alive_count_change = 0 && not_alive_count_change = 0\n");
     st->weirdness = true;
   }
   ddsrt_mutex_unlock (&st->lock);
@@ -970,7 +956,7 @@ static unsigned get_and_check_status (dds_entity_t reader, dds_entity_t writer_a
   rc = dds_get_publication_matched_status(writer_active, &pstatus);
   CU_ASSERT_FATAL(rc == DDS_RETCODE_OK);
   CU_ASSERT_FATAL(lstatus.alive_count + lstatus.not_alive_count <= 2);
-  printf ("sub %d | alive %d | not-alive %d | pub %d | data %d\n", (int)sstatus.current_count, (int)lstatus.alive_count, (int)lstatus.not_alive_count, (int)pstatus.current_count, dstatus != 0);
+  tprintf ("sub %d | alive %d | not-alive %d | pub %d | data %d\n", (int)sstatus.current_count, (int)lstatus.alive_count, (int)lstatus.not_alive_count, (int)pstatus.current_count, dstatus != 0);
   if (dstatus)
     result |= STATUS_DATA;
   if (sstatus.current_count == 2 && lstatus.not_alive_count == 2 && pstatus.current_count == 1)
@@ -1069,10 +1055,10 @@ static void wait_for_notalive (dds_entity_t reader, struct liveliness_changed_st
   dds_return_t rc;
   rc = dds_get_liveliness_changed_status(reader, &lstatus);
   CU_ASSERT_FATAL(rc == DDS_RETCODE_OK);
-  printf("early liveliness changed status: alive %"PRIu32" not-alive %"PRIu32"\n", lstatus.alive_count, lstatus.not_alive_count);
+  tprintf("early liveliness changed status: alive %"PRIu32" not-alive %"PRIu32"\n", lstatus.alive_count, lstatus.not_alive_count);
 
   ddsrt_mutex_lock (&listener_state->lock);
-  printf("early w0 %"PRIx64" alive %"PRIu32" not-alive %"PRIu32"\n", listener_state->w0_handle, listener_state->w0_alive, listener_state->w0_not_alive);
+  tprintf("early w0 %"PRIx64" alive %"PRIu32" not-alive %"PRIu32"\n", listener_state->w0_handle, listener_state->w0_alive, listener_state->w0_not_alive);
   CU_ASSERT_FATAL(!listener_state->weirdness);
   CU_ASSERT_FATAL(listener_state->w0_handle != 0);
   while (listener_state->w0_not_alive < listener_state->w0_alive && retries-- > 0)
@@ -1084,8 +1070,8 @@ static void wait_for_notalive (dds_entity_t reader, struct liveliness_changed_st
     ddsrt_mutex_lock(&listener_state->lock);
   }
 
-  printf("late liveliness changed status: alive %"PRIu32" not-alive %"PRIu32"\n", lstatus.alive_count, lstatus.not_alive_count);
-  printf("final w0 %"PRIx64" alive %"PRIu32" not-alive %"PRIu32"\n", listener_state->w0_handle, listener_state->w0_alive, listener_state->w0_not_alive);
+  tprintf("late liveliness changed status: alive %"PRIu32" not-alive %"PRIu32"\n", lstatus.alive_count, lstatus.not_alive_count);
+  tprintf("final w0 %"PRIx64" alive %"PRIu32" not-alive %"PRIu32"\n", listener_state->w0_handle, listener_state->w0_alive, listener_state->w0_not_alive);
   CU_ASSERT_FATAL(listener_state->w0_alive == listener_state->w0_not_alive);
   ddsrt_mutex_unlock(&listener_state->lock);
 }
@@ -1147,7 +1133,7 @@ static void lease_duration_zero_or_one_impl (dds_duration_t sleep, dds_livelines
     else
       exp_alive = nsamples - nsamples / 5; /* if sleeping, expect the vast majority (80%) of the writes to toggle liveliness */
     ddsrt_mutex_lock(&listener_state.lock);
-    printf("check w0_alive %"PRIu32" >= %"PRIu32"\n", listener_state.w0_alive, exp_alive);
+    tprintf("check w0_alive %"PRIu32" >= %"PRIu32"\n", listener_state.w0_alive, exp_alive);
     CU_ASSERT_FATAL(listener_state.w0_alive >= exp_alive);
     ddsrt_mutex_unlock(&listener_state.lock);
   }
@@ -1176,7 +1162,7 @@ CU_Test(ddsc_liveliness, lease_duration_zero_or_one, .init = liveliness_init, .f
           dds_duration_t s = sleep[sleep_idx];
           dds_liveliness_kind_t k = lkind[lkind_idx];
           dds_duration_t d = ldur[ldur_idx];
-          printf ("### lease_duration_zero_or_one: sleep = %"PRId64" lkind = %d ldur = %"PRId64" reader = %s\n", s, (int) k, d, rrd ? "remote" : "local");
+          tprintf ("### lease_duration_zero_or_one: sleep = %"PRId64" lkind = %d ldur = %"PRId64" reader = %s\n", s, (int) k, d, rrd ? "remote" : "local");
           lease_duration_zero_or_one_impl (s, k, d, rrd);
           printf ("\n");
         }
