@@ -214,9 +214,10 @@ struct ddsi_tran_factory
 };
 
 enum ddsi_tran_qos_purpose {
-  DDSI_TRAN_QOS_XMIT,
-  DDSI_TRAN_QOS_RECV_UC,
-  DDSI_TRAN_QOS_RECV_MC
+  DDSI_TRAN_QOS_XMIT_UC, // will send unicast only
+  DDSI_TRAN_QOS_XMIT_MC, // may send unicast or multicast
+  DDSI_TRAN_QOS_RECV_UC, // will be used for receiving unicast
+  DDSI_TRAN_QOS_RECV_MC  // will be used for receiving multicast
 };
 
 struct ddsi_tran_qos
@@ -244,7 +245,7 @@ DDS_INLINE_EXPORT inline uint32_t ddsi_receive_buffer_size (const struct ddsi_tr
 }
 DDS_INLINE_EXPORT inline dds_return_t ddsi_factory_create_conn (ddsi_tran_conn_t *conn, ddsi_tran_factory_t factory, uint32_t port, const struct ddsi_tran_qos *qos) {
   *conn = NULL;
-  if ((qos->m_interface != NULL) != (qos->m_purpose == DDSI_TRAN_QOS_XMIT))
+  if ((qos->m_interface != NULL) != (qos->m_purpose == DDSI_TRAN_QOS_XMIT_UC || qos->m_purpose == DDSI_TRAN_QOS_XMIT_MC))
     return DDS_RETCODE_BAD_PARAMETER;
   if (!ddsi_is_valid_port (factory, port))
     return DDS_RETCODE_BAD_PARAMETER;
