@@ -19,7 +19,7 @@
 #include "dds/ddsrt/hopscotch.h"
 #include "dds/ddsrt/avl.h"
 #include "dds/ddsi/ddsi_entity_index.h"
-#include "dds/ddsi/q_config.h"
+#include "dds/ddsi/ddsi_config_impl.h"
 #include "dds/ddsi/ddsi_domaingv.h"
 #include "dds/ddsi/q_entity.h"
 #include "dds/ddsi/q_gc.h"
