@@ -11,7 +11,7 @@
  */
 #include "ddsi_eth.h"
 #include "dds/ddsi/q_protocol.h" // for NN_LOCATOR_KIND_...
-#include "dds/ddsi/q_config.h" // for transport_selector
+#include "dds/ddsi/ddsi_config_impl.h" // for transport_selector
 
 int ddsi_eth_enumerate_interfaces (ddsi_tran_factory_t fact, enum ddsi_transport_selector transport_selector, ddsrt_ifaddrs_t **ifs)
 {
