@@ -13,7 +13,7 @@
 #define DDSI_ETH_H
 
 #include "dds/ddsi/ddsi_tran.h"
-#include "dds/ddsi/q_config.h"
+#include "dds/ddsi/ddsi_config_impl.h"
 
 #if defined (__cplusplus)
 extern "C" {
