@@ -14,7 +14,7 @@
 
 #include "dds/ddsrt/static_assert.h"
 #include "dds/ddsi/ddsi_portmapping.h"
-#include "dds/ddsi/q_config.h"
+#include "dds/ddsi/ddsi_config_impl.h"
 
 static bool get_port_int (uint32_t *port, const struct ddsi_portmapping *map, enum ddsi_port which, uint32_t domain_id, int32_t participant_index, char *str_if_overflow, size_t strsize)
 {
