@@ -19,7 +19,7 @@
 #include "dds/ddsrt/md5.h"
 #include "dds/ddsrt/mh3.h"
 #include "dds/ddsi/q_bswap.h"
-#include "dds/ddsi/q_config.h"
+#include "dds/ddsi/ddsi_config_impl.h"
 #include "dds/ddsi/q_freelist.h"
 #include "dds/ddsi/ddsi_tkmap.h"
 #include "dds/ddsi/ddsi_cdrstream.h"
@@ -27,18 +27,9 @@
 #include "dds/ddsi/ddsi_domaingv.h"
 #include "dds/ddsi/ddsi_serdata_default.h"
 #ifdef DDS_HAS_SHM
+#include "dds/ddsi/ddsi_shm_transport.h"
 #include "dds/ddsi/q_xmsg.h"
-#include "dds/ddsi/shm_sync.h"
-#endif
-
-#if DDSRT_ENDIAN == DDSRT_LITTLE_ENDIAN
-#define NATIVE_ENCODING CDR_LE
-#define NATIVE_ENCODING_PL PL_CDR_LE
-#elif DDSRT_ENDIAN == DDSRT_BIG_ENDIAN
-#define NATIVE_ENCODING CDR_BE
-#define NATIVE_ENCODING_PL PL_CDR_BE
-#else
-#error "DDSRT_ENDIAN neither LITTLE nor BIG"
+#include "iceoryx_binding_c/chunk.h"
 #endif
 
 /* 8k entries in the freelist seems to be roughly the amount needed to send
@@ -110,12 +101,16 @@ static void serdata_default_append_blob (struct ddsi_serdata_default **d, size_t
   memcpy (p, data, sz);
 }
 
+static const unsigned char *serdata_default_keybuf(const struct ddsi_serdata_default *d)
+{
+  assert(d->key.buftype != KEYBUFTYPE_UNSET);
+  return (d->key.buftype == KEYBUFTYPE_STATIC) ? d->key.u.stbuf : d->key.u.dynbuf;
+}
+
 static struct ddsi_serdata *fix_serdata_default(struct ddsi_serdata_default *d, uint32_t basehash)
 {
-  if (d->keyhash.m_iskey)
-    d->c.hash = ddsrt_mh3 (d->keyhash.m_hash, 16, 0) ^ basehash;
-  else
-    d->c.hash = *((uint32_t *)d->keyhash.m_hash) ^ basehash;
+  assert (d->key.keysize > 0); // we use a different function for implementing the keyless case
+  d->c.hash = ddsrt_mh3 (serdata_default_keybuf(d), d->key.keysize, basehash); // FIXME: or the full buffer, regardless of actual size?
   return &d->c;
 }
 
@@ -135,18 +130,8 @@ static bool serdata_default_eqkey(const struct ddsi_serdata *acmn, const struct
 {
   const struct ddsi_serdata_default *a = (const struct ddsi_serdata_default *)acmn;
   const struct ddsi_serdata_default *b = (const struct ddsi_serdata_default *)bcmn;
-  assert (a->keyhash.m_set);
-#if 0
-  char astr[50], bstr[50];
-  for (int i = 0; i < 16; i++) {
-    sprintf (astr + 3*i, ":%02x", (unsigned char)a->keyhash.m_hash[i]);
-  }
-  for (int i = 0; i < 16; i++) {
-    sprintf (bstr + 3*i, ":%02x", (unsigned char)b->keyhash.m_hash[i]);
-  }
-  printf("serdata_default_eqkey: %s %s\n", astr+1, bstr+1);
-#endif
-  return memcmp (a->keyhash.m_hash, b->keyhash.m_hash, 16) == 0;
+  assert (a->key.buftype != KEYBUFTYPE_UNSET && b->key.buftype != KEYBUFTYPE_UNSET);
+  return a->key.keysize == b->key.keysize && memcmp (serdata_default_keybuf(a), serdata_default_keybuf(b), a->key.keysize) == 0;
 }
 
 static bool serdata_default_eqkey_nokey (const struct ddsi_serdata *acmn, const struct ddsi_serdata *bcmn)
@@ -160,37 +145,31 @@ static void serdata_default_free(struct ddsi_serdata *dcmn)
   struct ddsi_serdata_default *d = (struct ddsi_serdata_default *)dcmn;
   assert(ddsrt_atomic_ld32(&d->c.refc) == 0);
 
+  if (d->key.buftype == KEYBUFTYPE_DYNALLOC)
+    ddsrt_free(d->key.u.dynbuf);
+
 #ifdef DDS_HAS_SHM
-  //ICEORYX_TODO the chunk is released concurrently to iox_sub_take_chunk here,
-  //                 we will need mutex protection for this call
-  //                 when is the free called exactly?
-  if (d->c.iox_chunk)
-  {
-    iox_sub_t *sub = d->c.iox_subscriber;
-    shm_lock_iox_sub(*sub);
-    iox_sub_release_chunk(*sub, d->c.iox_chunk);
-    d->c.iox_chunk = NULL;
-    shm_unlock_iox_sub(*sub);
-  }
+  free_iox_chunk(d->c.iox_subscriber, &d->c.iox_chunk);
 #endif
 
   if (d->size > MAX_SIZE_FOR_POOL || !nn_freelist_push (&d->serpool->freelist, d))
     dds_free (d);
 }
 
-static void serdata_default_init(struct ddsi_serdata_default *d, const struct ddsi_sertype_default *tp, enum ddsi_serdata_kind kind)
+static void serdata_default_init(struct ddsi_serdata_default *d, const struct ddsi_sertype_default *tp, enum ddsi_serdata_kind kind, uint32_t xcdr_version)
 {
   ddsi_serdata_init (&d->c, &tp->c, kind);
   d->pos = 0;
 #ifndef NDEBUG
   d->fixed = false;
 #endif
-  d->hdr.identifier = tp->native_encoding_identifier;
+  if (xcdr_version != CDR_ENC_VERSION_UNDEF)
+    d->hdr.identifier = ddsi_sertype_get_native_enc_identifier (xcdr_version, tp->encoding_format);
+  else
+    d->hdr.identifier = 0;
   d->hdr.options = 0;
-  memset (d->keyhash.m_hash, 0, sizeof (d->keyhash.m_hash));
-  d->keyhash.m_set = 0;
-  d->keyhash.m_iskey = 0;
-  d->keyhash.m_keysize = 0;
+  d->key.buftype = KEYBUFTYPE_UNSET;
+  d->key.keysize = 0;
 }
 
 static struct ddsi_serdata_default *serdata_default_allocnew (struct serdatapool *serpool, uint32_t init_size)
@@ -201,20 +180,116 @@ static struct ddsi_serdata_default *serdata_default_allocnew (struct serdatapool
   return d;
 }
 
-static struct ddsi_serdata_default *serdata_default_new_size (const struct ddsi_sertype_default *tp, enum ddsi_serdata_kind kind, uint32_t size)
+static struct ddsi_serdata_default *serdata_default_new_size (const struct ddsi_sertype_default *tp, enum ddsi_serdata_kind kind, uint32_t size, uint32_t xcdr_version)
 {
   struct ddsi_serdata_default *d;
   if (size <= MAX_SIZE_FOR_POOL && (d = nn_freelist_pop (&tp->serpool->freelist)) != NULL)
     ddsrt_atomic_st32 (&d->c.refc, 1);
   else if ((d = serdata_default_allocnew (tp->serpool, size)) == NULL)
     return NULL;
-  serdata_default_init (d, tp, kind);
+  serdata_default_init (d, tp, kind, xcdr_version);
   return d;
 }
 
-static struct ddsi_serdata_default *serdata_default_new (const struct ddsi_sertype_default *tp, enum ddsi_serdata_kind kind)
+static struct ddsi_serdata_default *serdata_default_new (const struct ddsi_sertype_default *tp, enum ddsi_serdata_kind kind, uint32_t xcdr_version)
 {
-  return serdata_default_new_size (tp, kind, DEFAULT_NEW_SIZE);
+  return serdata_default_new_size (tp, kind, DEFAULT_NEW_SIZE, xcdr_version);
+}
+
+static inline bool is_valid_xcdr_id (unsigned short cdr_identifier)
+{
+  /* PL_CDR_(L|B)E version 1 only supported for discovery data, using ddsi_serdata_plist */
+  return (cdr_identifier == CDR_LE || cdr_identifier == CDR_BE
+    || cdr_identifier == CDR2_LE || cdr_identifier == CDR2_BE
+    || cdr_identifier == D_CDR2_LE || cdr_identifier == D_CDR2_BE
+    || cdr_identifier == PL_CDR2_LE || cdr_identifier == PL_CDR2_BE);
+}
+
+enum gen_serdata_key_input_kind {
+  GSKIK_SAMPLE,
+  GSKIK_CDRSAMPLE,
+  GSKIK_CDRKEY
+};
+
+static inline bool is_topic_fixed_key(uint32_t flagset, uint32_t xcdrv)
+{
+  if (xcdrv == CDR_ENC_VERSION_1)
+    return flagset & DDS_TOPIC_FIXED_KEY;
+  else if (xcdrv == CDR_ENC_VERSION_2)
+    return flagset & DDS_TOPIC_FIXED_KEY_XCDR2;
+  assert (0);
+  return false;
+}
+
+static bool gen_serdata_key (const struct ddsi_sertype_default *type, struct ddsi_serdata_default_key *kh, enum gen_serdata_key_input_kind input_kind, void *input)
+{
+  const struct ddsi_sertype_default_desc *desc = &type->type;
+  struct dds_istream *is = NULL;
+  kh->buftype = KEYBUFTYPE_UNSET;
+  if (desc->keys.nkeys == 0)
+  {
+    kh->buftype = KEYBUFTYPE_STATIC;
+    kh->keysize = 0;
+  }
+  else if (input_kind == GSKIK_CDRKEY)
+  {
+    is = input;
+    if (is->m_xcdr_version == CDR_ENC_VERSION_2)
+    {
+      kh->buftype = KEYBUFTYPE_DYNALIAS;
+      assert (is->m_size < (1u << 30));
+      kh->keysize = is->m_size & SERDATA_DEFAULT_KEYSIZE_MASK;
+      kh->u.dynbuf = (unsigned char *) is->m_buffer;
+    }
+  }
+
+  if (kh->buftype == KEYBUFTYPE_UNSET)
+  {
+    // Force the key in the serdata object to be serialized in XCDR2 format
+    dds_ostream_t os;
+    dds_ostream_init (&os, 0, CDR_ENC_VERSION_2);
+    if (is_topic_fixed_key(desc->flagset, CDR_ENC_VERSION_2))
+    {
+      // FIXME: there are more cases where we don't have to allocate memory
+      os.m_buffer = kh->u.stbuf;
+      os.m_size = DDS_FIXED_KEY_MAX_SIZE;
+    }
+    switch (input_kind)
+    {
+      case GSKIK_SAMPLE:
+        dds_stream_write_key (&os, input, type);
+        break;
+      case GSKIK_CDRSAMPLE:
+        if (!dds_stream_extract_key_from_data (input, &os, type))
+          return false;
+        break;
+      case GSKIK_CDRKEY:
+        assert (is);
+        assert (is->m_xcdr_version == CDR_ENC_VERSION_1);
+        dds_stream_extract_key_from_key (is, &os, type);
+        break;
+    }
+    assert (os.m_index < (1u << 30));
+    kh->keysize = os.m_index & SERDATA_DEFAULT_KEYSIZE_MASK;
+    if (is_topic_fixed_key (desc->flagset, CDR_ENC_VERSION_2))
+      kh->buftype = KEYBUFTYPE_STATIC;
+    else
+    {
+      kh->buftype = KEYBUFTYPE_DYNALLOC;
+      kh->u.dynbuf = ddsrt_realloc (os.m_buffer, os.m_index); // don't waste bytes FIXME: maybe should be willing to waste a little
+    }
+  }
+  return true;
+}
+
+static bool gen_serdata_key_from_sample (const struct ddsi_sertype_default *type, struct ddsi_serdata_default_key *kh, const char *sample)
+{
+  return gen_serdata_key (type, kh, GSKIK_SAMPLE, (void *) sample);
+}
+
+static bool gen_serdata_key_from_cdr (dds_istream_t * __restrict is, struct ddsi_serdata_default_key * __restrict kh, const struct ddsi_sertype_default * __restrict type, const bool just_key)
+{
+  return gen_serdata_key (type, kh, just_key ? GSKIK_CDRKEY : GSKIK_CDRSAMPLE, is);
 }
 
 /* Construct a serdata from a fragchain received over the network */
@@ -228,7 +303,7 @@ static struct ddsi_serdata_default *serdata_default_from_ser_common (const struc
      serdata */
   if (size > UINT32_MAX - offsetof (struct ddsi_serdata_default, hdr))
     return NULL;
-  struct ddsi_serdata_default *d = serdata_default_new_size (tp, kind, (uint32_t) size);
+  struct ddsi_serdata_default *d = serdata_default_new_size (tp, kind, (uint32_t) size, CDR_ENC_VERSION_UNDEF);
   if (d == NULL)
     return NULL;
 
@@ -238,7 +313,8 @@ static struct ddsi_serdata_default *serdata_default_from_ser_common (const struc
   assert (fragchain->maxp1 >= off); /* CDR header must be in first fragment */
 
   memcpy (&d->hdr, NN_RMSG_PAYLOADOFF (fragchain->rmsg, NN_RDATA_PAYLOAD_OFF (fragchain)), sizeof (d->hdr));
-  assert (d->hdr.identifier == CDR_LE || d->hdr.identifier == CDR_BE);
+  if (!is_valid_xcdr_id (d->hdr.identifier))
+    goto err;
 
   while (fragchain)
   {
@@ -254,26 +330,34 @@ static struct ddsi_serdata_default *serdata_default_from_ser_common (const struc
     fragchain = fragchain->nextfrag;
   }
 
-  const bool needs_bswap = (d->hdr.identifier != NATIVE_ENCODING);
-  d->hdr.identifier = NATIVE_ENCODING;
+  const bool needs_bswap = !CDR_ENC_IS_NATIVE (d->hdr.identifier);
+  d->hdr.identifier = CDR_ENC_TO_NATIVE (d->hdr.identifier);
   const uint32_t pad = ddsrt_fromBE2u (d->hdr.options) & 2;
-  if (d->pos < pad)
-  {
-    ddsi_serdata_unref (&d->c);
-    return NULL;
-  }
-  else if (!dds_stream_normalize (d->data, d->pos - pad, needs_bswap, tp, kind == SDK_KEY))
-  {
-    ddsi_serdata_unref (&d->c);
-    return NULL;
-  }
-  else
-  {
-    dds_istream_t is;
-    dds_istream_from_serdata_default (&is, d);
-    dds_stream_extract_keyhash (&is, &d->keyhash, tp, kind == SDK_KEY);
-    return d;
-  }
+  const uint32_t xcdr_version = ddsi_sertype_enc_id_xcdr_version (d->hdr.identifier);
+  const uint32_t encoding_format = ddsi_sertype_enc_id_enc_format (d->hdr.identifier);
+  if (encoding_format != tp->encoding_format)
+    goto err;
+
+  uint32_t actual_size;
+  if (d->pos < pad || !dds_stream_normalize (d->data, d->pos - pad, needs_bswap, xcdr_version, tp, kind == SDK_KEY, &actual_size))
+    goto err;
+
+  dds_istream_t is;
+  dds_istream_init (&is, actual_size, d->data, xcdr_version);
+  if (!gen_serdata_key_from_cdr (&is, &d->key, tp, kind == SDK_KEY))
+    goto err;
+  // for (int n = 0; n < d->key.keysize; n++) {
+  //   if (d->key.buftype == KEYBUFTYPE_DYNALLOC || d->key.buftype == KEYBUFTYPE_DYNALIAS)
+  //     printf("%02x ", d->key.u.dynbuf[n]);
+  //   else
+  //     printf("%02x ", d->key.u.stbuf[n]);
+  // }
+  // printf("\n");
+  return d;
+
+err:
+  ddsi_serdata_unref (&d->c);
+  return NULL;
 }
 
 static struct ddsi_serdata_default *serdata_default_from_ser_iov_common (const struct ddsi_sertype *tpcmn, enum ddsi_serdata_kind kind, ddsrt_msg_iovlen_t niov, const ddsrt_iovec_t *iov, size_t size)
@@ -289,36 +373,38 @@ static struct ddsi_serdata_default *serdata_default_from_ser_iov_common (const s
   assert (niov >= 1);
   if (iov[0].iov_len < 4) /* CDR header */
     return NULL;
-  struct ddsi_serdata_default *d = serdata_default_new_size (tp, kind, (uint32_t) size);
+  struct ddsi_serdata_default *d = serdata_default_new_size (tp, kind, (uint32_t) size, CDR_ENC_VERSION_UNDEF);
   if (d == NULL)
     return NULL;
 
   memcpy (&d->hdr, iov[0].iov_base, sizeof (d->hdr));
-  assert (d->hdr.identifier == CDR_LE || d->hdr.identifier == CDR_BE);
+  if (!is_valid_xcdr_id (d->hdr.identifier))
+    goto err;
   serdata_default_append_blob (&d, iov[0].iov_len - 4, (const char *) iov[0].iov_base + 4);
   for (ddsrt_msg_iovlen_t i = 1; i < niov; i++)
     serdata_default_append_blob (&d, iov[i].iov_len, iov[i].iov_base);
 
-  const bool needs_bswap = (d->hdr.identifier != NATIVE_ENCODING);
-  d->hdr.identifier = NATIVE_ENCODING;
+  const bool needs_bswap = !CDR_ENC_IS_NATIVE (d->hdr.identifier);
+  d->hdr.identifier = CDR_ENC_TO_NATIVE (d->hdr.identifier);
   const uint32_t pad = ddsrt_fromBE2u (d->hdr.options) & 2;
-  if (d->pos < pad)
-  {
-    ddsi_serdata_unref (&d->c);
-    return NULL;
-  }
-  else if (!dds_stream_normalize (d->data, d->pos - pad, needs_bswap, tp, kind == SDK_KEY))
-  {
-    ddsi_serdata_unref (&d->c);
-    return NULL;
-  }
-  else
-  {
-    dds_istream_t is;
-    dds_istream_from_serdata_default (&is, d);
-    dds_stream_extract_keyhash (&is, &d->keyhash, tp, kind == SDK_KEY);
-    return d;
-  }
+  const uint32_t xcdr_version = ddsi_sertype_enc_id_xcdr_version (d->hdr.identifier);
+  const uint32_t encoding_format = ddsi_sertype_enc_id_enc_format (d->hdr.identifier);
+  if (encoding_format != tp->encoding_format)
+    goto err;
+
+  uint32_t actual_size;
+  if (d->pos < pad || !dds_stream_normalize (d->data, d->pos - pad, needs_bswap, xcdr_version, tp, kind == SDK_KEY, &actual_size))
+    goto err;
+
+  dds_istream_t is;
+  dds_istream_init (&is, actual_size, d->data, xcdr_version);
+  if (!gen_serdata_key_from_cdr (&is, &d->key, tp, kind == SDK_KEY))
+    goto err;
+  return d;
+
+err:
+  ddsi_serdata_unref (&d->c);
+  return NULL;
 }
 
 static struct ddsi_serdata *serdata_default_from_ser (const struct ddsi_sertype *tpcmn, enum ddsi_serdata_kind kind, const struct nn_rdata *fragchain, size_t size)
@@ -355,188 +441,196 @@ static struct ddsi_serdata *serdata_default_from_ser_iov_nokey (const struct dds
 
 static struct ddsi_serdata *ddsi_serdata_from_keyhash_cdr (const struct ddsi_sertype *tpcmn, const ddsi_keyhash_t *keyhash)
 {
-  /* FIXME: not quite sure this is correct, though a check against a specially hacked OpenSplice suggests it is */
   const struct ddsi_sertype_default *tp = (const struct ddsi_sertype_default *)tpcmn;
-  if (!(tp->type.flagset & DDS_TOPIC_FIXED_KEY))
+  if (!is_topic_fixed_key (tp->type.flagset, CDR_ENC_VERSION_2))
   {
     /* keyhash is MD5 of a key value, so impossible to turn into a key value */
     return NULL;
   }
   else
   {
-    struct ddsi_serdata_default *d = serdata_default_new(tp, SDK_KEY);
-    if (d == NULL)
-      return NULL;
-    serdata_default_append_blob (&d, sizeof (keyhash->value), keyhash->value);
-    DDSRT_WARNING_MSVC_OFF(6326)
-    if (!dds_stream_normalize (d->data, d->pos, (NATIVE_ENCODING != CDR_BE), tp, true))
-    {
-      ddsi_serdata_unref (&d->c);
-      return NULL;
-    }
-    DDSRT_WARNING_MSVC_ON(6326)
-    memcpy (d->keyhash.m_hash, keyhash->value, sizeof (d->keyhash.m_hash));
-    d->keyhash.m_set = 1;
-    d->keyhash.m_iskey = 1;
-    d->keyhash.m_keysize = sizeof (d->keyhash.m_hash);
-    return fix_serdata_default(d, tp->c.serdata_basehash);
+    const ddsrt_iovec_t iovec[2] = {
+      { .iov_base = (unsigned char[]) { 0,0,0,0 }, .iov_len = 4}, // big-endian, unspecified padding
+      { .iov_base = (void *) keyhash->value, .iov_len = (ddsrt_iov_len_t) sizeof (*keyhash) }
+    };
+    return serdata_default_from_ser_iov (tpcmn, SDK_KEY, 2, iovec, 4 + sizeof (*keyhash));
   }
 }
 
 static struct ddsi_serdata *ddsi_serdata_from_keyhash_cdr_nokey (const struct ddsi_sertype *tpcmn, const ddsi_keyhash_t *keyhash)
 {
   const struct ddsi_sertype_default *tp = (const struct ddsi_sertype_default *)tpcmn;
-  struct ddsi_serdata_default *d = serdata_default_new(tp, SDK_KEY);
+  /* For keyless topic, the CDR encoding version is irrelevant */
+  struct ddsi_serdata_default *d = serdata_default_new(tp, SDK_KEY, CDR_ENC_VERSION_UNDEF);
   if (d == NULL)
     return NULL;
   (void)keyhash;
-  d->keyhash.m_set = 1;
-  d->keyhash.m_iskey = 1;
-  d->keyhash.m_keysize = 0;
   return fix_serdata_default_nokey(d, tp->c.serdata_basehash);
 }
 
-static void gen_keyhash_from_sample (const struct ddsi_sertype_default *type, dds_keyhash_t *kh, const char *sample)
-{
-  const struct ddsi_sertype_default_desc *desc = &type->type;
-  kh->m_set = 1;
-  if (desc->keys.nkeys == 0)
-  {
-    kh->m_iskey = 1;
-    kh->m_keysize = 0;
-  }
-  else if (desc->flagset & DDS_TOPIC_FIXED_KEY)
-  {
-    dds_ostreamBE_t os;
-    kh->m_iskey = 1;
-    kh->m_keysize = sizeof(kh->m_hash);
-    dds_ostreamBE_init (&os, 0);
-    os.x.m_buffer = kh->m_hash;
-    os.x.m_size = 16;
-    dds_stream_write_keyBE (&os, sample, type);
-  }
-  else
-  {
-    dds_ostreamBE_t os;
-    ddsrt_md5_state_t md5st;
-    kh->m_iskey = 0;
-    kh->m_keysize = sizeof(kh->m_hash);
-    dds_ostreamBE_init (&os, 64);
-    dds_stream_write_keyBE (&os, sample, type);
-    ddsrt_md5_init (&md5st);
-    ddsrt_md5_append (&md5st, os.x.m_buffer, os.x.m_index);
-    ddsrt_md5_finish (&md5st, kh->m_hash);
-    dds_ostreamBE_fini (&os);
-  }
-}
-
 #ifdef DDS_HAS_SHM
-static struct ddsi_serdata* serdata_default_from_iox(const struct ddsi_sertype* tpcmn, enum ddsi_serdata_kind kind, void* sub, void* iox_buffer)
+static struct ddsi_serdata* serdata_default_from_received_iox_buffer(const struct ddsi_sertype* tpcmn, enum ddsi_serdata_kind kind, void* sub, void* iox_buffer)
 {
-  iceoryx_header_t* ice_hdr = (iceoryx_header_t*)iox_buffer;
+  const iceoryx_header_t* ice_hdr = iceoryx_header_from_chunk(iox_buffer);
 
   const struct ddsi_sertype_default* tp = (const struct ddsi_sertype_default*)tpcmn;
 
-  struct ddsi_serdata_default* d = serdata_default_new_size(tp, kind, ice_hdr->data_size);
+  struct ddsi_serdata_default *d = serdata_default_new_size (tp, kind, ice_hdr->data_size, tp->write_encoding_version);
 
-  //ICEORYX_TODO: we do no copy here but store the pointer to the chunk
-  //              the pointer was gotten in some concurrent thread of the shm_monitor
-  //              problems may arise due to concurrent iox_release_chunk
+  // note: we do not deserialize or memcpy here, just take ownership of the chunk
   d->c.iox_chunk = iox_buffer;
   d->c.iox_subscriber = sub;
-  memcpy(d->keyhash.m_hash, ice_hdr->keyhash.value, 16);
-  d->keyhash.m_set = 1;
-  d->keyhash.m_iskey = 0;  //if set to 1, will cause issues @ serdata_default_to_untyped at the endianness swap
-  d->keyhash.m_keysize = 16;
+  d->key.buftype = KEYBUFTYPE_STATIC;
+  d->key.keysize = DDS_FIXED_KEY_MAX_SIZE;
+  memcpy(d->key.u.stbuf, ice_hdr->keyhash.value, DDS_FIXED_KEY_MAX_SIZE);
+
   fix_serdata_default(d, tpcmn->serdata_basehash);
 
   return (struct ddsi_serdata*)d;
 }
+
+// Creates a serdata for the case where only iceoryx is required (i.e. no network).
+// This skips expensive serialization and just takes ownership of the iceoryx buffer.
+// Computing the keyhash is currently still required.
+static struct ddsi_serdata *ddsi_serdata_default_from_loaned_sample (const struct ddsi_sertype *type, enum ddsi_serdata_kind kind, const char *sample)
+{
+  const struct ddsi_sertype_default *t = (const struct ddsi_sertype_default *)type;
+  struct ddsi_serdata_default *d = serdata_default_new (t, kind, t->write_encoding_version);
+
+  if(d == NULL)
+    return NULL;
+
+  // Currently needed even in the shared memory case (since it is potentially used at the reader side).
+  // This may still incur computational costs linear in the sample size (?).
+  // TODO: Can we avoid this with specific handling on the reader side which does not require the keyhash?
+  if (!gen_serdata_key_from_sample (t, &d->key, sample))
+    return NULL;
+
+  struct ddsi_serdata *serdata = &d->c;
+  serdata->iox_chunk = (void*) sample;
+  return serdata;
+}
+
+static struct ddsi_serdata* serdata_default_from_iox(const struct ddsi_sertype* tpcmn, enum ddsi_serdata_kind kind, void* sub, void* buffer)
+{
+  if (sub == NULL)
+    return ddsi_serdata_default_from_loaned_sample(tpcmn, kind, buffer);
+  else
+    return serdata_default_from_received_iox_buffer(tpcmn, kind, sub, buffer);
+}
 #endif
 
-static struct ddsi_serdata_default *serdata_default_from_sample_cdr_common (const struct ddsi_sertype *tpcmn, enum ddsi_serdata_kind kind, const void *sample)
+
+static struct ddsi_serdata_default *serdata_default_from_sample_cdr_common (const struct ddsi_sertype *tpcmn, enum ddsi_serdata_kind kind, uint32_t xcdr_version, const void *sample)
 {
   const struct ddsi_sertype_default *tp = (const struct ddsi_sertype_default *)tpcmn;
-  struct ddsi_serdata_default *d = serdata_default_new(tp, kind);
+  struct ddsi_serdata_default *d = serdata_default_new(tp, kind, xcdr_version);
   if (d == NULL)
     return NULL;
+
   dds_ostream_t os;
-  gen_keyhash_from_sample (tp, &d->keyhash, sample);
   dds_ostream_from_serdata_default (&os, d);
   switch (kind)
   {
     case SDK_EMPTY:
+      dds_ostream_add_to_serdata_default (&os, &d);
       break;
     case SDK_KEY:
       dds_stream_write_key (&os, sample, tp);
+      dds_ostream_add_to_serdata_default (&os, &d);
+
+      /* FIXME: detect cases where the XCDR1 and 2 representations are equal,
+         so that we could alias the XCDR1 key from d->data */
+      /* FIXME: use CDR encoding version used by the writer, not the write encoding
+         of the sertype */
+      if (tp->write_encoding_version == CDR_ENC_VERSION_2)
+      {
+        d->key.buftype = KEYBUFTYPE_DYNALIAS;
+        // dds_ostream_add_to_serdata_default pads the size to a multiple of 4,
+        // writing the number of padding bytes added in the 2 least-significant
+        // bits of options (when interpreted as a big-endian 16-bit number) in
+        // accordance with the XTypes spec.
+        //
+        // Those padding bytes are not part of the key!
+        assert (ddsrt_fromBE2u (d->hdr.options) < 4);
+        d->key.keysize = (d->pos - ddsrt_fromBE2u (d->hdr.options)) & SERDATA_DEFAULT_KEYSIZE_MASK;
+        d->key.u.dynbuf = (unsigned char *) d->data;
+      }
+      else
+      {
+        /* We have a XCDR1 key, so this must be converted to XCDR2 to store
+           it as key in this serdata. */
+        if (!gen_serdata_key_from_sample (tp, &d->key, sample))
+          return NULL;
+      }
       break;
     case SDK_DATA:
-      dds_stream_write_sample (&os, sample, tp);
+      if (!dds_stream_write_sample (&os, sample, tp))
+        return NULL;
+      dds_ostream_add_to_serdata_default (&os, &d);
+      if (!gen_serdata_key_from_sample (tp, &d->key, sample))
+        return NULL;
       break;
   }
-  dds_ostream_add_to_serdata_default (&os, &d);
   return d;
 }
 
+static struct ddsi_serdata *serdata_default_from_sample_data_representation (const struct ddsi_sertype *tpcmn, enum ddsi_serdata_kind kind, dds_data_representation_id_t data_representation, const void *sample, bool key)
+{
+  assert (data_representation == DDS_DATA_REPRESENTATION_XCDR1 || data_representation == DDS_DATA_REPRESENTATION_XCDR2);
+  struct ddsi_serdata_default *d;
+  uint32_t xcdr_version = data_representation == DDS_DATA_REPRESENTATION_XCDR1 ? CDR_ENC_VERSION_1 : CDR_ENC_VERSION_2;
+  if ((d = serdata_default_from_sample_cdr_common (tpcmn, kind, xcdr_version, sample)) == NULL)
+    return NULL;
+  return key ? fix_serdata_default (d, tpcmn->serdata_basehash) : fix_serdata_default_nokey (d, tpcmn->serdata_basehash);
+}
+
 static struct ddsi_serdata *serdata_default_from_sample_cdr (const struct ddsi_sertype *tpcmn, enum ddsi_serdata_kind kind, const void *sample)
 {
-  struct ddsi_serdata_default *d;
-  if ((d = serdata_default_from_sample_cdr_common (tpcmn, kind, sample)) == NULL)
-    return NULL;
-  return fix_serdata_default (d, tpcmn->serdata_basehash);
+  return serdata_default_from_sample_data_representation (tpcmn, kind, DDS_DATA_REPRESENTATION_XCDR1, sample, true);
+}
+
+static struct ddsi_serdata *serdata_default_from_sample_xcdr2 (const struct ddsi_sertype *tpcmn, enum ddsi_serdata_kind kind, const void *sample)
+{
+  return serdata_default_from_sample_data_representation (tpcmn, kind, DDS_DATA_REPRESENTATION_XCDR2, sample, true);
 }
 
 static struct ddsi_serdata *serdata_default_from_sample_cdr_nokey (const struct ddsi_sertype *tpcmn, enum ddsi_serdata_kind kind, const void *sample)
 {
-  struct ddsi_serdata_default *d;
-  if ((d = serdata_default_from_sample_cdr_common (tpcmn, kind, sample)) == NULL)
-    return NULL;
-  return fix_serdata_default_nokey (d, tpcmn->serdata_basehash);
+  return serdata_default_from_sample_data_representation (tpcmn, kind, DDS_DATA_REPRESENTATION_XCDR1, sample, false);
 }
 
+static struct ddsi_serdata *serdata_default_from_sample_xcdr2_nokey (const struct ddsi_sertype *tpcmn, enum ddsi_serdata_kind kind, const void *sample)
+{
+  return serdata_default_from_sample_data_representation (tpcmn, kind, DDS_DATA_REPRESENTATION_XCDR2, sample, false);
+}
+
+
 static struct ddsi_serdata *serdata_default_to_untyped (const struct ddsi_serdata *serdata_common)
 {
   const struct ddsi_serdata_default *d = (const struct ddsi_serdata_default *)serdata_common;
   const struct ddsi_sertype_default *tp = (const struct ddsi_sertype_default *)d->c.type;
-  assert (d->hdr.identifier == NATIVE_ENCODING || d->hdr.identifier == NATIVE_ENCODING_PL);
-  struct ddsi_serdata_default *d_tl = serdata_default_new(tp, SDK_KEY);
+
+  assert (CDR_ENC_IS_NATIVE (d->hdr.identifier));
+  struct ddsi_serdata_default *d_tl = serdata_default_new (tp, SDK_KEY, CDR_ENC_VERSION_2);
   if (d_tl == NULL)
     return NULL;
   d_tl->c.type = NULL;
   d_tl->c.hash = d->c.hash;
   d_tl->c.timestamp.v = INT64_MIN;
-  d_tl->keyhash = d->keyhash;
   /* These things are used for the key-to-instance map and only subject to eq, free and conversion to an invalid
      sample of some type for topics that can end up in a RHC, so, of the four kinds we have, only for CDR-with-key
      the payload is of interest. */
-  if (d->c.ops == &ddsi_serdata_ops_cdr)
+  if (d->c.ops == &ddsi_serdata_ops_cdr || d->c.ops == &ddsi_serdata_ops_xcdr2)
   {
-    assert (d->hdr.identifier == NATIVE_ENCODING);
-    if (d->c.kind == SDK_KEY)
-      serdata_default_append_blob (&d_tl, d->pos, d->data);
-    else if (d->keyhash.m_iskey)
-    {
-      serdata_default_append_blob (&d_tl, sizeof (d->keyhash.m_hash), d->keyhash.m_hash);
-#if NATIVE_ENCODING != CDR_BE
-      bool ok = dds_stream_normalize (d_tl->data, d_tl->pos, true, tp, true);
-      assert (ok);
-      (void) ok;
-#endif
-    }
-    else
-    {
-      dds_istream_t is;
-      dds_ostream_t os;
-      dds_istream_from_serdata_default (&is, d);
-      dds_ostream_from_serdata_default (&os, d_tl);
-      dds_stream_extract_key_from_data (&is, &os, tp);
-      if (os.m_index < os.m_size)
-      {
-        os.m_buffer = dds_realloc (os.m_buffer, os.m_index);
-        os.m_size = os.m_index;
-      }
-      dds_ostream_add_to_serdata_default (&os, &d_tl);
-    }
+    serdata_default_append_blob (&d_tl, d->key.keysize, serdata_default_keybuf (d));
+    d_tl->key.buftype = KEYBUFTYPE_DYNALIAS;
+    d_tl->key.keysize = d->key.keysize;
+    d_tl->key.u.dynbuf = (unsigned char *) d_tl->data;
+  }
+  else
+  {
+    assert (d->c.ops == &ddsi_serdata_ops_cdr_nokey || d->c.ops == &ddsi_serdata_ops_xcdr2_nokey);
   }
   return (struct ddsi_serdata *)d_tl;
 }
@@ -570,17 +664,29 @@ static bool serdata_default_to_sample_cdr (const struct ddsi_serdata *serdata_co
 {
   const struct ddsi_serdata_default *d = (const struct ddsi_serdata_default *)serdata_common;
   const struct ddsi_sertype_default *tp = (const struct ddsi_sertype_default *) d->c.type;
+  dds_istream_t is;
 #ifdef DDS_HAS_SHM
   if (d->c.iox_chunk)
   {
-    iceoryx_header_t* hdr = (iceoryx_header_t*)d->c.iox_chunk;
-    memcpy(sample, SHIFT_PAST_ICEORYX_HEADER(hdr), hdr->data_size);
+    void* iox_chunk = d->c.iox_chunk;
+    iceoryx_header_t* hdr = iceoryx_header_from_chunk(iox_chunk);
+    if(hdr->shm_data_state == IOX_CHUNK_CONTAINS_SERIALIZED_DATA) {
+      dds_istream_init (&is, hdr->data_size, iox_chunk, ddsi_sertype_enc_id_xcdr_version(d->hdr.identifier));
+      assert (CDR_ENC_IS_NATIVE (d->hdr.identifier));
+      if (d->c.kind == SDK_KEY)
+        dds_stream_read_key (&is, sample, tp);
+      else
+        dds_stream_read_sample (&is, sample, tp);
+    } else {
+      // should contain raw unserialized data
+      // we could check the data_state but should not be needed
+      memcpy(sample, iox_chunk, hdr->data_size);
+    }
     return true;
   }
 #endif
-  dds_istream_t is;
   if (bufptr) abort(); else { (void)buflim; } /* FIXME: haven't implemented that bit yet! */
-  assert (d->hdr.identifier == NATIVE_ENCODING);
+  assert (CDR_ENC_IS_NATIVE (d->hdr.identifier));
   dds_istream_from_serdata_default(&is, d);
   if (d->c.kind == SDK_KEY)
     dds_stream_read_key (&is, sample, tp);
@@ -597,7 +703,7 @@ static bool serdata_default_untyped_to_sample_cdr (const struct ddsi_sertype *se
   assert (d->c.type == NULL);
   assert (d->c.kind == SDK_KEY);
   assert (d->c.ops == sertype_common->serdata_ops);
-  assert (d->hdr.identifier == NATIVE_ENCODING);
+  assert (CDR_ENC_IS_NATIVE (d->hdr.identifier));
   if (bufptr) abort(); else { (void)buflim; } /* FIXME: haven't implemented that bit yet! */
   dds_istream_from_serdata_default(&is, d);
   dds_stream_read_key (&is, sample, tp);
@@ -627,19 +733,52 @@ static size_t serdata_default_print_cdr (const struct ddsi_sertype *sertype_comm
 static void serdata_default_get_keyhash (const struct ddsi_serdata *serdata_common, struct ddsi_keyhash *buf, bool force_md5)
 {
   const struct ddsi_serdata_default *d = (const struct ddsi_serdata_default *)serdata_common;
+  const struct ddsi_sertype_default *tp = (const struct ddsi_sertype_default *)d->c.type;
   assert(buf);
-  assert(d->keyhash.m_set);
-  if (force_md5 && d->keyhash.m_iskey /* m_iskey == !md5 */)
+  assert(d->key.buftype != KEYBUFTYPE_UNSET);
+
+  // Convert native representation to what keyhashes expect
+  // d->key could also be in big-endian, but that eliminates the possibility of aliasing d->data
+  // on little endian machines and forces a double copy of the key to be present for SDK_KEY
+  //
+  // As nobody should be using the DDSI keyhash, the price to pay for the conversion looks like
+  // a price worth paying
+
+  uint32_t xcdrv = ddsi_sertype_enc_id_xcdr_version (d->hdr.identifier);
+
+  /* serdata has a XCDR2 serialized key, so initializer the istream with this version
+     and with the size of that key (d->key.keysize) */
+  dds_istream_t is;
+  dds_istream_init (&is, d->key.keysize, serdata_default_keybuf(d), CDR_ENC_VERSION_2);
+
+  /* The output stream uses the XCDR version from the serdata, so that the keyhash in
+     ostream is calculated using this CDR representation (XTypes spec 7.6.8, RTPS spec 9.6.3.8) */
+  dds_ostreamBE_t os;
+  dds_ostreamBE_init (&os, 0, xcdrv);
+  dds_stream_extract_keyBE_from_key (&is, &os, tp);
+  assert (is.m_index == d->key.keysize);
+
+  /* We know the key size for XCDR2 encoding, but for XCDR1 there can be additional
+     padding because of 8-byte alignment of key fields */
+  if (xcdrv == CDR_ENC_VERSION_2)
+    assert (os.x.m_index == d->key.keysize);
+
+  /* Cannot use is_topic_fixed_key here, because in case there is a bounded string
+     key field, it may contain a shorter string and fit in the 16 bytes */
+  uint32_t actual_keysz = os.x.m_index;
+  if (force_md5 || actual_keysz > DDS_FIXED_KEY_MAX_SIZE)
   {
     ddsrt_md5_state_t md5st;
-    ddsrt_md5_init  (&md5st);
-    ddsrt_md5_append(&md5st, (ddsrt_md5_byte_t*)(d->keyhash.m_hash), d->keyhash.m_keysize);
-    ddsrt_md5_finish(&md5st, (ddsrt_md5_byte_t*)(buf->value));
+    ddsrt_md5_init (&md5st);
+    ddsrt_md5_append (&md5st, (ddsrt_md5_byte_t *) os.x.m_buffer, actual_keysz);
+    ddsrt_md5_finish (&md5st, (ddsrt_md5_byte_t *) buf->value);
   }
   else
   {
-    memcpy (buf->value, d->keyhash.m_hash, 16);
+    memset (buf->value, 0, DDS_FIXED_KEY_MAX_SIZE);
+    memcpy (buf->value, os.x.m_buffer, actual_keysz);
   }
+  dds_ostreamBE_fini (&os);
 }
 
 const struct ddsi_serdata_ops ddsi_serdata_ops_cdr = {
@@ -664,6 +803,28 @@ const struct ddsi_serdata_ops ddsi_serdata_ops_cdr = {
 #endif
 };
 
+const struct ddsi_serdata_ops ddsi_serdata_ops_xcdr2 = {
+  .get_size = serdata_default_get_size,
+  .eqkey = serdata_default_eqkey,
+  .free = serdata_default_free,
+  .from_ser = serdata_default_from_ser,
+  .from_ser_iov = serdata_default_from_ser_iov,
+  .from_keyhash = ddsi_serdata_from_keyhash_cdr,
+  .from_sample = serdata_default_from_sample_xcdr2,
+  .to_ser = serdata_default_to_ser,
+  .to_sample = serdata_default_to_sample_cdr,
+  .to_ser_ref = serdata_default_to_ser_ref,
+  .to_ser_unref = serdata_default_to_ser_unref,
+  .to_untyped = serdata_default_to_untyped,
+  .untyped_to_sample = serdata_default_untyped_to_sample_cdr,
+  .print = serdata_default_print_cdr,
+  .get_keyhash = serdata_default_get_keyhash
+#ifdef DDS_HAS_SHM
+  , .get_sample_size = ddsi_serdata_iox_size
+  , .from_iox_buffer = serdata_default_from_iox
+#endif
+};
+
 const struct ddsi_serdata_ops ddsi_serdata_ops_cdr_nokey = {
   .get_size = serdata_default_get_size,
   .eqkey = serdata_default_eqkey_nokey,
@@ -685,3 +846,25 @@ const struct ddsi_serdata_ops ddsi_serdata_ops_cdr_nokey = {
   , .from_iox_buffer = serdata_default_from_iox
 #endif
 };
+
+const struct ddsi_serdata_ops ddsi_serdata_ops_xcdr2_nokey = {
+  .get_size = serdata_default_get_size,
+  .eqkey = serdata_default_eqkey_nokey,
+  .free = serdata_default_free,
+  .from_ser = serdata_default_from_ser_nokey,
+  .from_ser_iov = serdata_default_from_ser_iov_nokey,
+  .from_keyhash = ddsi_serdata_from_keyhash_cdr_nokey,
+  .from_sample = serdata_default_from_sample_xcdr2_nokey,
+  .to_ser = serdata_default_to_ser,
+  .to_sample = serdata_default_to_sample_cdr,
+  .to_ser_ref = serdata_default_to_ser_ref,
+  .to_ser_unref = serdata_default_to_ser_unref,
+  .to_untyped = serdata_default_to_untyped,
+  .untyped_to_sample = serdata_default_untyped_to_sample_cdr_nokey,
+  .print = serdata_default_print_cdr,
+  .get_keyhash = serdata_default_get_keyhash
+#ifdef DDS_HAS_SHM
+  , .get_sample_size = ddsi_serdata_iox_size
+  , .from_iox_buffer = serdata_default_from_iox
+#endif
+};
