@@ -19,7 +19,7 @@
 #include "dds/ddsrt/ifaddrs.h"
 #include "dds/ddsi/ddsi_tran.h"
 #include "dds/ddsi/ddsi_ipaddr.h"
-#include "dds/ddsi/q_config.h"
+#include "dds/ddsi/ddsi_config_impl.h"
 #include "dds/ddsi/q_log.h"
 #include "dds/ddsi/ddsi_domaingv.h"
 
