@@ -14,7 +14,7 @@
 
 #include "dds/ddsrt/endian.h"
 #include "dds/ddsi/q_log.h"
-#include "dds/ddsi/q_config.h"
+#include "dds/ddsi/ddsi_config_impl.h"
 #include "dds/ddsi/ddsi_domaingv.h"
 #include "dds/ddsi/q_bswap.h"
 #include "dds/ddsi/q_pcap.h"
