@@ -12,8 +12,9 @@
 #include <string.h>
 #include <assert.h>
 
+#include "dds/features.h"
 #include "dds/ddsi/ddsi_xqos.h"
-#include "dds/ddsi/ddsi_typeid.h"
+#include "dds/ddsi/ddsi_typelib.h"
 #include "dds/ddsi/ddsi_typelookup.h"
 #include "dds/ddsi/ddsi_domaingv.h"
 #include "dds/ddsi/q_misc.h"
@@ -47,7 +48,7 @@ static int partitions_match_default (const dds_qos_t *x)
   return 0;
 }
 
-int partitions_match_p (const dds_qos_t *a, const dds_qos_t *b)
+static int partitions_match_p (const dds_qos_t *a, const dds_qos_t *b)
 {
   if (!(a->present & QP_PARTITION) || a->partition.n == 0)
     return partitions_match_default (b);
@@ -68,47 +69,59 @@ int partitions_match_p (const dds_qos_t *a, const dds_qos_t *b)
 
 #ifdef DDS_HAS_TYPE_DISCOVERY
 
-static bool check_assignability (struct tl_meta *rd_tlm, struct tl_meta *wr_tlm) ddsrt_nonnull_all;
-
-static bool check_assignability (struct tl_meta *rd_tlm, struct tl_meta *wr_tlm)
-{
-  assert (rd_tlm->sertype != NULL);
-  assert (wr_tlm->sertype != NULL);
-  return ddsi_sertype_assignable_from (rd_tlm->sertype, wr_tlm->sertype);
-}
-
-static bool check_endpoint_typeid (struct ddsi_domaingv *gv, const type_identifier_t *type_id, struct tl_meta **tlm, bool *req_lookup)
+static bool is_endpoint_type_resolved (struct ddsi_domaingv *gv, char *type_name, const ddsi_type_pair_t *type_pair, bool *req_lookup, const char *entity)
   ddsrt_nonnull((1, 2, 3));
 
-static bool check_endpoint_typeid (struct ddsi_domaingv *gv, const type_identifier_t *type_id, struct tl_meta **tlm, bool *req_lookup)
+static bool is_endpoint_type_resolved (struct ddsi_domaingv *gv, char *type_name, const ddsi_type_pair_t *type_pair, bool *req_lookup, const char *entity)
 {
-  assert (tlm != NULL);
-
-  // type_id = NULL is treated the same as type_id = 0...0, so this implies type_id may not be a null pointer
-  assert (!ddsi_typeid_none (type_id));
-
-  ddsrt_mutex_lock (&gv->tl_admin_lock);
-  /* no refcounting for returned tlm object, but its lifetime is
-     at least that of the endpoint that refers to it */
-  *tlm = ddsi_tl_meta_lookup_locked (gv, type_id);
-  assert (*tlm != NULL);
-  if ((*tlm)->state != TL_META_RESOLVED)
+  assert (type_pair);
+  ddsrt_mutex_lock (&gv->typelib_lock);
+  if (!ddsi_type_pair_has_minimal_obj (type_pair) && !ddsi_type_pair_has_complete_obj (type_pair))
   {
-    GVTRACE ("typeid unresolved "PTYPEIDFMT"\n", PTYPEID(*type_id));
+    struct ddsi_typeid_str str;
+    const ddsi_typeid_t *tid_m = ddsi_type_pair_minimal_id (type_pair),
+      *tid_c = ddsi_type_pair_complete_id (type_pair);
+    GVTRACE ("unresolved %s type %s ", entity, type_name);
+    if (tid_m)
+      GVTRACE ("min %s", ddsi_make_typeid_str (&str, tid_m));
+    if (tid_c)
+      GVTRACE ("compl %s", ddsi_make_typeid_str (&str, tid_c));
+    GVTRACE ("\n");
     /* defer requesting unresolved type until after the endpoint qos lock
        has been released, so just set a bool value indicating that a type
        lookup is required */
     if (req_lookup != NULL)
       *req_lookup = true;
-    ddsrt_mutex_unlock (&gv->tl_admin_lock);
+    ddsrt_mutex_unlock (&gv->typelib_lock);
     return false;
   }
-  ddsrt_mutex_unlock (&gv->tl_admin_lock);
+  ddsrt_mutex_unlock (&gv->typelib_lock);
   return true;
 }
 
 #endif /* DDS_HAS_TYPE_DISCOVERY */
 
+static int data_representation_match_p (const dds_qos_t *rd_qos, const dds_qos_t *wr_qos)
+{
+  assert (rd_qos->present & QP_DATA_REPRESENTATION);
+  assert (rd_qos->data_representation.value.n > 0);
+  assert (wr_qos->present & QP_DATA_REPRESENTATION);
+  assert (wr_qos->data_representation.value.n > 0);
+
+  /* For the writer only use the first representation identifier and ignore 1..n (spec 7.6.3.1.1) */
+  for (uint32_t i = 0; i < rd_qos->data_representation.value.n; i++)
+    if (rd_qos->data_representation.value.ids[i] == wr_qos->data_representation.value.ids[0])
+      return 1;
+  return 0;
+}
+
+#ifdef DDS_HAS_TYPE_DISCOVERY
+static bool type_pair_has_id (const ddsi_type_pair_t *pair)
+{
+  return pair && (pair->minimal || pair->complete);
+}
+#endif
+
 bool qos_match_mask_p (
     struct ddsi_domaingv *gv,
     const dds_qos_t *rd_qos,
@@ -116,8 +129,8 @@ bool qos_match_mask_p (
     uint64_t mask,
     dds_qos_policy_id_t *reason
 #ifdef DDS_HAS_TYPE_DISCOVERY
-    , const type_identifier_t *rd_typeid
-    , const type_identifier_t *wr_typeid
+    , const struct ddsi_type_pair *rd_type_pair
+    , const struct ddsi_type_pair *wr_type_pair
     , bool *rd_typeid_req_lookup
     , bool *wr_typeid_req_lookup
 #endif
@@ -125,50 +138,23 @@ bool qos_match_mask_p (
 {
   DDSRT_UNUSED_ARG (gv);
 #ifndef NDEBUG
-  unsigned musthave = (QP_RXO_MASK | QP_PARTITION | QP_TOPIC_NAME | QP_TYPE_NAME) & mask;
+  uint64_t musthave = (QP_RXO_MASK | QP_PARTITION | QP_TOPIC_NAME | QP_TYPE_NAME | QP_DATA_REPRESENTATION) & mask;
   assert ((rd_qos->present & musthave) == musthave);
   assert ((wr_qos->present & musthave) == musthave);
 #endif
 
-  mask &= rd_qos->present & wr_qos->present;
-  *reason = DDS_INVALID_QOS_POLICY_ID;
-  if ((mask & QP_TOPIC_NAME) && strcmp (rd_qos->topic_name, wr_qos->topic_name) != 0)
-    return false;
-  if ((mask & QP_TYPE_NAME) && strcmp (rd_qos->type_name, wr_qos->type_name) != 0)
-    return false;
-
 #ifdef DDS_HAS_TYPE_DISCOVERY
   if (rd_typeid_req_lookup != NULL)
     *rd_typeid_req_lookup = false;
   if (wr_typeid_req_lookup != NULL)
     *wr_typeid_req_lookup = false;
-
-  if (ddsi_typeid_none (rd_typeid) || ddsi_typeid_none (wr_typeid))
-  {
-    // Type id missing on either or both: automatic failure if "force type validation"
-    // is set.  If it is missing for one, there is no point in requesting it for the
-    // other (it wouldn't be inspected anyway).
-    if (rd_qos->type_consistency.force_type_validation)
-    {
-      *reason = DDS_TYPE_CONSISTENCY_ENFORCEMENT_QOS_POLICY_ID;
-      return false;
-    }
-  }
-  else
-  {
-    struct tl_meta *rd_tlm = NULL, *wr_tlm = NULL;
-    if (!check_endpoint_typeid (gv, rd_typeid, &rd_tlm, rd_typeid_req_lookup))
-      return false;
-    if (!check_endpoint_typeid (gv, wr_typeid, &wr_tlm, wr_typeid_req_lookup))
-      return false;
-    if (!check_assignability (rd_tlm, wr_tlm))
-    {
-      *reason = DDS_TYPE_CONSISTENCY_ENFORCEMENT_QOS_POLICY_ID;
-      return false;
-    }
-  }
 #endif
 
+  mask &= rd_qos->present & wr_qos->present;
+  *reason = DDS_INVALID_QOS_POLICY_ID;
+  if ((mask & QP_TOPIC_NAME) && strcmp (rd_qos->topic_name, wr_qos->topic_name) != 0)
+    return false;
+
   if ((mask & QP_RELIABILITY) && rd_qos->reliability.kind > wr_qos->reliability.kind) {
     *reason = DDS_RELIABILITY_QOS_POLICY_ID;
     return false;
@@ -217,6 +203,64 @@ bool qos_match_mask_p (
     *reason = DDS_PARTITION_QOS_POLICY_ID;
     return false;
   }
+  if ((mask & QP_DATA_REPRESENTATION) && !data_representation_match_p (rd_qos, wr_qos)) {
+    *reason = DDS_DATA_REPRESENTATION_QOS_POLICY_ID;
+    return false;
+  }
+
+#ifdef DDS_HAS_TYPE_DISCOVERY
+  if (!type_pair_has_id (rd_type_pair) || !type_pair_has_id (wr_type_pair))
+  {
+    // Type info missing on either or both: automatic failure if "force type validation"
+    // is set.  If it is missing for one, there is no point in requesting it for the
+    // other (it wouldn't be inspected anyway).
+    if (rd_qos->type_consistency.force_type_validation)
+    {
+      *reason = DDS_TYPE_CONSISTENCY_ENFORCEMENT_QOS_POLICY_ID;
+      return false;
+    }
+    // If either the reader or writer does not provide a type id, the type names are consulted
+    // (XTypes spec 7.6.3.4.2)
+    if ((mask & QP_TYPE_NAME) && strcmp (rd_qos->type_name, wr_qos->type_name) != 0)
+      return false;
+  }
+  else
+  {
+    dds_type_consistency_enforcement_qospolicy_t tce = {
+      .kind = DDS_TYPE_CONSISTENCY_ALLOW_TYPE_COERCION,
+      .ignore_sequence_bounds = true,
+      .ignore_string_bounds = true,
+      .ignore_member_names = false,
+      .prevent_type_widening = false,
+      .force_type_validation = false
+    };
+    (void) dds_qget_type_consistency (rd_qos, &tce.kind, &tce.ignore_sequence_bounds, &tce.ignore_string_bounds, &tce.ignore_member_names, &tce.prevent_type_widening, &tce.force_type_validation);
+
+    if (tce.kind == DDS_TYPE_CONSISTENCY_DISALLOW_TYPE_COERCION)
+    {
+      if (ddsi_typeid_compare (ddsi_type_pair_minimal_id (rd_type_pair), ddsi_type_pair_minimal_id (wr_type_pair)))
+      {
+        *reason = DDS_TYPE_CONSISTENCY_ENFORCEMENT_QOS_POLICY_ID;
+        return false;
+      }
+    }
+    else
+    {
+      if (!is_endpoint_type_resolved (gv, rd_qos->type_name, rd_type_pair, rd_typeid_req_lookup, "rd")
+          || !is_endpoint_type_resolved (gv, wr_qos->type_name, wr_type_pair, wr_typeid_req_lookup, "wr"))
+        return false;
+      if (!ddsi_is_assignable_from (gv, rd_type_pair, wr_type_pair, &tce))
+      {
+        *reason = DDS_TYPE_CONSISTENCY_ENFORCEMENT_QOS_POLICY_ID;
+        return false;
+      }
+    }
+  }
+#else
+  if ((mask & QP_TYPE_NAME) && strcmp (rd_qos->type_name, wr_qos->type_name) != 0)
+    return false;
+#endif
+
   return true;
 }
 
@@ -226,8 +270,8 @@ bool qos_match_p (
     const dds_qos_t *wr_qos,
     dds_qos_policy_id_t *reason
 #ifdef DDS_HAS_TYPE_DISCOVERY
-    , const type_identifier_t *rd_typeid
-    , const type_identifier_t *wr_typeid
+    , const struct ddsi_type_pair *rd_type_pair
+    , const struct ddsi_type_pair *wr_type_pair
     , bool *rd_typeid_req_lookup
     , bool *wr_typeid_req_lookup
 #endif
@@ -235,7 +279,7 @@ bool qos_match_p (
 {
   dds_qos_policy_id_t dummy;
 #ifdef DDS_HAS_TYPE_DISCOVERY
-  return qos_match_mask_p (gv, rd_qos, wr_qos, ~(uint64_t)0, reason ? reason : &dummy, rd_typeid, wr_typeid, rd_typeid_req_lookup, wr_typeid_req_lookup);
+  return qos_match_mask_p (gv, rd_qos, wr_qos, ~(uint64_t)0, reason ? reason : &dummy, rd_type_pair, wr_type_pair, rd_typeid_req_lookup, wr_typeid_req_lookup);
 #else
   return qos_match_mask_p (gv, rd_qos, wr_qos, ~(uint64_t)0, reason ? reason : &dummy);
 #endif
