@@ -67,6 +67,12 @@ DDS_INLINE_EXPORT inline int64_t ddsrt_bswap8 (int64_t x)
 #define ddsrt_toBE4u(x) ddsrt_bswap4u (x)
 #define ddsrt_toBE8(x) ddsrt_bswap8 (x)
 #define ddsrt_toBE8u(x) ddsrt_bswap8u (x)
+#define ddsrt_toLE2(x) (x)
+#define ddsrt_toLE2u(x) (x)
+#define ddsrt_toLE4(x) (x)
+#define ddsrt_toLE4u(x) (x)
+#define ddsrt_toLE8(x) (x)
+#define ddsrt_toLE8u(x) (x)
 #define ddsrt_toBO2(bo, x) ((bo) == DDSRT_BOSEL_BE ? ddsrt_bswap2 (x) : (x))
 #define ddsrt_toBO2u(bo, x) ((bo) == DDSRT_BOSEL_BE ? ddsrt_bswap2u (x) : (x))
 #define ddsrt_toBO4(bo, x) ((bo) == DDSRT_BOSEL_BE ? ddsrt_bswap4 (x) : (x))
@@ -85,6 +91,12 @@ DDS_INLINE_EXPORT inline int64_t ddsrt_bswap8 (int64_t x)
 #define ddsrt_toBE4u(x) (x)
 #define ddsrt_toBE8(x) (x)
 #define ddsrt_toBE8u(x) (x)
+#define ddsrt_toLE2(x) ddsrt_bswap2 (x)
+#define ddsrt_toLE2u(x) ddsrt_bswap2u (x)
+#define ddsrt_toLE4(x) ddsrt_bswap4 (x)
+#define ddsrt_toLE4u(x) ddsrt_bswap4u (x)
+#define ddsrt_toLE8(x) ddsrt_bswap8 (x)
+#define ddsrt_toLE8u(x) ddsrt_bswap8u (x)
 #define ddsrt_toBO2(bo, x) ((bo) == DDSRT_BOSEL_LE ? ddsrt_bswap2 (x) : (x))
 #define ddsrt_toBO2u(bo, x) ((bo) == DDSRT_BOSEL_LE ? ddsrt_bswap2u (x) : (x))
 #define ddsrt_toBO4(bo, x) ((bo) == DDSRT_BOSEL_LE ? ddsrt_bswap4 (x) : (x))
