@@ -158,17 +158,27 @@ ddsrt_rwlock_init (ddsrt_rwlock_t *rwlock)
 {
   assert(rwlock != NULL);
 
+#if __SunOS_5_6
+  if (pthread_mutex_init(&rwlock->rwlock, NULL) != 0)
+    abort();
+#else
   /* process-shared attribute is set to PTHREAD_PROCESS_PRIVATE by default */
   if (pthread_rwlock_init(&rwlock->rwlock, NULL) != 0)
     abort();
+#endif
 }
 
 void
 ddsrt_rwlock_destroy (ddsrt_rwlock_t *rwlock)
 {
   assert(rwlock != NULL);
-  if (pthread_rwlock_destroy (&rwlock->rwlock) != 0)
+#if __SunOS_5_6
+  if (pthread_mutex_destroy(&rwlock->rwlock) != 0)
     abort();
+#else
+  if (pthread_rwlock_destroy(&rwlock->rwlock) != 0)
+    abort();
+#endif
 }
 
 void ddsrt_rwlock_read (ddsrt_rwlock_t *rwlock)
@@ -176,7 +186,11 @@ void ddsrt_rwlock_read (ddsrt_rwlock_t *rwlock)
   int err;
 
   assert(rwlock != NULL);
+#if __SunOS_5_6
+  err = pthread_mutex_lock(&rwlock->rwlock);
+#else
   err = pthread_rwlock_rdlock(&rwlock->rwlock);
+#endif
   assert(err == 0);
   (void)err;
 }
@@ -186,7 +200,11 @@ void ddsrt_rwlock_write (ddsrt_rwlock_t *rwlock)
   int err;
 
   assert(rwlock != NULL);
+#if __SunOS_5_6
+  err = pthread_mutex_lock(&rwlock->rwlock);
+#else
   err = pthread_rwlock_wrlock(&rwlock->rwlock);
+#endif
   assert(err == 0);
   (void)err;
 }
@@ -196,7 +214,11 @@ bool ddsrt_rwlock_tryread (ddsrt_rwlock_t *rwlock)
   int err;
 
   assert(rwlock != NULL);
+#if __SunOS_5_6
+  err = pthread_mutex_trylock(&rwlock->rwlock);
+#else
   err = pthread_rwlock_tryrdlock(&rwlock->rwlock);
+#endif
   assert(err == 0 || err == EBUSY);
   return err == 0;
 }
@@ -206,7 +228,11 @@ bool ddsrt_rwlock_trywrite (ddsrt_rwlock_t *rwlock)
   int err;
 
   assert(rwlock != NULL);
+#if __SunOS_5_6
+  err = pthread_mutex_trylock(&rwlock->rwlock);
+#else
   err = pthread_rwlock_trywrlock(&rwlock->rwlock);
+#endif
   assert(err == 0 || err == EBUSY);
 
   return err == 0;
@@ -217,7 +243,11 @@ void ddsrt_rwlock_unlock (ddsrt_rwlock_t *rwlock)
   int err;
 
   assert(rwlock != NULL);
+#if __SunOS_5_6
+  err = pthread_mutex_unlock(&rwlock->rwlock);
+#else
   err = pthread_rwlock_unlock(&rwlock->rwlock);
+#endif
   assert(err == 0);
   (void)err;
 }
